from plone.app.content.browser.contents import ContentsBaseAction
from plone.base import PloneMessageFactory as _


class SetDefaultPageActionView(ContentsBaseAction):
    success_msg = _("Default page set successfully")
    failure_msg = _("Failed to set default page")

    def __call__(self):
        cid = self.request.form.get("id")
        self.errors = []

        if cid not in self.context.objectIds():
            self.errors.append(
                _(
                    "There is no object with short name " "${name} in this folder.",
                    mapping={"name": cid},
                )
            )
        else:
            self.context.setDefaultPage(cid)
        return self.message()



from plone.app.contenttypes.testing import PLONE_APP_CONTENTTYPES_FIXTURE
from plone.app.testing import applyProfile
from plone.app.testing import FunctionalTesting
from plone.app.testing import IntegrationTesting
from plone.app.testing import PLONE_FIXTURE
from plone.app.testing import PloneSandboxLayer
from plone.app.testing import TEST_USER_PASSWORD
from Products.CMFCore.utils import getToolByName
from zope.configuration import xmlconfig
from zope.interface import implementer
from zope.interface import provider
from zope.schema.interfaces import IVocabularyFactory
from zope.schema.vocabulary import SimpleTerm
from zope.schema.vocabulary import SimpleVocabulary

import doctest


@implementer(IVocabularyFactory)
class ExampleVocabulary:
    def __call__(self, context, query=None):
        items = ["One", "Two", "Three"]
        tmp = SimpleVocabulary(
            [
                SimpleTerm(it.lower(), it.lower(), it)
                for it in items
                if query is None or query.lower() in it.lower()
            ]
        )
        tmp.test = 1
        return tmp


@provider(IVocabularyFactory)
def ExampleFunctionVocabulary(context, query=None):
    items = ["First", "Second", "Third"]
    tmp = SimpleVocabulary(
        [
            SimpleTerm(it.lower(), it.lower(), it)
            for it in items
            if query is None or query.lower() in it.lower()
        ]
    )
    return tmp


class PloneAppContent(PloneSandboxLayer):

    defaultBases = (PLONE_FIXTURE,)

    USER_NAME = "johndoe"
    USER_PASSWORD = TEST_USER_PASSWORD
    MEMBER_NAME = "janedoe"
    MEMBER_PASSWORD = TEST_USER_PASSWORD
    USER_WITH_FULLNAME_NAME = "jim"
    USER_WITH_FULLNAME_FULLNAME = "Jim Fulton"
    USER_WITH_FULLNAME_PASSWORD = TEST_USER_PASSWORD
    MANAGER_USER_NAME = "manager"
    MANAGER_USER_PASSWORD = TEST_USER_PASSWORD

    def setUpZope(self, app, configurationContext):
        # Load ZCML
        import plone.app.content

        xmlconfig.file(
            "configure.zcml", plone.app.content, context=configurationContext
        )

    def setUpPloneSite(self, portal):
        # Creates some users
        acl_users = getToolByName(portal, "acl_users")
        acl_users.userFolderAddUser(
            self.USER_NAME,
            self.USER_PASSWORD,
            [],
            [],
        )
        acl_users.userFolderAddUser(
            self.MEMBER_NAME,
            self.MEMBER_PASSWORD,
            ["Member"],
            [],
        )
        acl_users.userFolderAddUser(
            self.USER_WITH_FULLNAME_NAME,
            self.USER_WITH_FULLNAME_PASSWORD,
            ["Member"],
            [],
        )
        mtool = getToolByName(portal, "portal_membership", None)
        mtool.addMember("jim", "Jim", ["Member"], [])
        mtool.getMemberById("jim").setMemberProperties(
            {"fullname": "Jim Fult\xc3\xb8rn"}
        )

        acl_users.userFolderAddUser(
            self.MANAGER_USER_NAME,
            self.MANAGER_USER_PASSWORD,
            ["Manager"],
            [],
        )
        portal.portal_workflow.setDefaultChain("simple_publication_workflow")


class NonAsciiLayer(PloneSandboxLayer):
    def setUpZope(self, app, configurationContext):
        import plone.app.content.tests

        xmlconfig.file(
            "profiles/non-ascii-workflow.zcml",
            plone.app.content.tests,
            context=configurationContext,
        )

    def setUpPloneSite(self, portal):
        # applyProfile which has non-ascii characters in state titles
        applyProfile(portal, "plone.app.content.tests:non-ascii-workflow")


PLONE_APP_CONTENT_FIXTURE = PloneAppContent()
PLONE_APP_CONTENT_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_APP_CONTENT_FIXTURE,), name="PloneAppContent:Integration"
)
PLONE_APP_CONTENT_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(PLONE_APP_CONTENT_FIXTURE,), name="PloneAppContent:Functional"
)


# Dexterity test layers
PLONE_APP_CONTENT_DX_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_APP_CONTENTTYPES_FIXTURE,), name="PloneAppContentDX:Integration"
)
PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING = FunctionalTesting(
    bases=(PLONE_APP_CONTENTTYPES_FIXTURE,), name="PloneAppContentDX:Functional"
)


# Test layer with a workflow containing non-ascii characters in state titles.
PLONE_APP_CONTENT_NON_ASCII_LAYER = NonAsciiLayer()
PLONE_APP_CONTENT_NON_ASCII_INTEGRATION_TESTING = IntegrationTesting(
    bases=(PLONE_APP_CONTENT_NON_ASCII_LAYER,),
    name="PloneAppContentNonAscii:Integration",
)


optionflags = (
    doctest.REPORT_ONLY_FIRST_FAILURE | doctest.ELLIPSIS | doctest.NORMALIZE_WHITESPACE
)



from plone.app.content.testing import PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING
from plone.app.testing import setRoles
from plone.app.testing import TEST_USER_ID
from plone.app.testing import TEST_USER_PASSWORD
from plone.testing.zope import Browser

import transaction
import unittest


FOLDER = {
    "id": "testfolder",
    "title": "Test Folder",
    "description": "Test Folder Description",
}

DOCUMENT = {
    "id": "testdoc",
    "title": "Test Document",
    "description": "Test Document Description",
}

NEWSITEM = {
    "id": "testnews",
    "title": "Test News Item",
    "description": "Test News Item Description",
}


class SelectDefaultPageDXTestCase(unittest.TestCase):

    layer = PLONE_APP_CONTENT_DX_FUNCTIONAL_TESTING

    def setUp(self):
        self.portal = self.layer["portal"]
        self.portal.acl_users.userFolderAddUser("editor", TEST_USER_PASSWORD, ["Editor"], [])

        self._create_structure()
        transaction.commit()

        self.browser = Browser(self.layer["app"])
        self.browser.addHeader(
            "Authorization", "Basic {}:{}".format("editor", TEST_USER_PASSWORD)
        )

    def tearDown(self):
        self.portal.manage_delObjects(ids=FOLDER["id"])
        transaction.commit()

    def _createFolder(self):
        self.portal.invokeFactory(id=FOLDER["id"], type_name="Folder")
        folder = getattr(self.portal, FOLDER["id"])
        folder.setTitle(FOLDER["title"])
        folder.setDescription(FOLDER["description"])
        folder.reindexObject()
        # we don't want it in the navigation
        # folder.setExcludeFromNav(True)
        return folder

    def _createDocument(self, context):
        context.invokeFactory(id=DOCUMENT["id"], type_name="Document")
        doc = getattr(context, DOCUMENT["id"])
        doc.setTitle(DOCUMENT["title"])
        doc.setDescription(DOCUMENT["description"])
        doc.reindexObject()
        # we don't want it in the navigation
        # doc.setExcludeFromNav(True)
        return doc

    def _createNewsItem(self, context):
        context.invokeFactory(id=NEWSITEM["id"], type_name="News Item")
        doc = getattr(context, NEWSITEM["id"])
        doc.setTitle(NEWSITEM["title"])
        doc.setDescription(NEWSITEM["description"])
        doc.reindexObject()
        # we don't want it in the navigation
        # doc.setExcludeFromNav(True)
        return doc

    def _create_structure(self):
        setRoles(self.portal, TEST_USER_ID, ["Manager"])
        folder = self._createFolder()
        self._createDocument(folder)
        return folder

    def test_select_default_page_view(self):
        """Check that the form can be rendered."""
        folder = self.portal.testfolder

        self.browser.open("%s/@@select_default_page" % folder.absolute_url())

        self.assertTrue("Select default page" in self.browser.contents)
        self.assertTrue('id="testdoc"' in self.browser.contents)

    def test_select_default_page_view_with_folderish_type(self):
        """Check if folderish types are available."""
        folder = self.portal.testfolder
        folder.invokeFactory(id=FOLDER["id"], type_name="Folder")
        folder2 = getattr(folder, FOLDER["id"])
        folder.setTitle(FOLDER["title"])
        folder2.reindexObject()
        folder_fti = self.portal.portal_types["Folder"]
        folder_fti.manage_changeProperties(
            filter_content_types=True, allowed_content_types=[]
        )
        view = folder.restrictedTraverse("@@select_default_page")()

        self.assertTrue('id="testdoc"' in view)
        self.assertTrue('id="testfolder"' in view)

    def test_default_page_action_cancel(self):
        """Check the Cancel action."""
        folder = self.portal.testfolder

        self.browser.open("%s/@@select_default_page" % folder.absolute_url())
        cancel_button = self.browser.getControl(name="form.buttons.Cancel")
        cancel_button.click()

        self.assertEqual(self.browser.url, folder.absolute_url())
        self.assertIs(folder.getDefaultPage(), None)

    def test_default_page_action_save(self):
        """Check the Save action."""
        folder = self.portal.testfolder
        self.browser.open("%s/@@select_default_page" % folder.absolute_url())

        submit_button = self.browser.getControl(name="form.buttons.Save")
        submit_button.click()

        self.assertEqual(self.browser.url, folder.absolute_url())
        self.assertEqual(folder.getDefaultPage(), "testdoc")

    def test_selectable_types_filter(self):
        self.portal.portal_registry["plone.default_page_types"] = ["News Item"]
        folder = self.portal.testfolder
        self._createNewsItem(folder)

        view = folder.restrictedTraverse("@@select_default_page")()
        self.assertTrue('id="testdoc"' not in view)
        self.assertTrue('id="testnews"' in view)



from AccessControl import Unauthorized
from Acquisition import aq_inner
from Acquisition import aq_parent
from plone.app.content.browser.file import TUS_ENABLED
from plone.app.content.browser.interfaces import IFolderContentsView
from plone.app.content.interfaces import IStructureAction
from plone.app.content.utils import json_dumps
from plone.app.content.utils import json_loads
from plone.app.uuid.utils import uuidToCatalogBrain
from plone.base import PloneMessageFactory as _
from plone.base import utils
from plone.base.interfaces.controlpanel import ISiteSchema
from plone.protect.postonly import check as checkpost
from plone.registry.interfaces import IRegistry
from plone.uuid.interfaces import IUUID
from Products.CMFCore.utils import getToolByName
from Products.Five import BrowserView
from Products.PortalTransforms.transforms.safe_html import SafeHTML
from zope.browsermenu.interfaces import IBrowserMenu
from zope.component import getMultiAdapter
from zope.component import getUtilitiesFor
from zope.component import getUtility
from zope.i18n import translate
from zope.interface import implementer
from zope.schema.interfaces import IVocabularyFactory


class ContentsBaseAction(BrowserView):

    success_msg = _("Success")
    failure_msg = _("Failure")
    required_obj_permission = None

    @property
    def site(self):
        return utils.get_top_site_from_url(self.context, self.request)

    def objectTitle(self, obj):
        context = aq_inner(obj)
        title = utils.pretty_title_or_id(context, context)
        return utils.safe_text(title)

    def protect(self):
        authenticator = getMultiAdapter(
            (self.context, self.request), name="authenticator"
        )
        if not authenticator.verify():
            raise Unauthorized
        checkpost(self.request)

    def json(self, data):
        self.request.response.setHeader(
            "Content-Type", "application/json; charset=utf-8"
        )
        return json_dumps(data)

    def get_selection(self):
        selection = self.request.form.get("selection", "[]")
        return json_loads(selection)

    def action(self, obj):
        """
        fill in this method to do action against each item in the selection
        """
        pass

    def finish(self):
        pass

    def __call__(self, keep_selection_order=False):
        self.protect()
        self.errors = []
        context = aq_inner(self.context)
        selection = self.get_selection()

        parts = str(self.request.form.get("folder", "").lstrip("/")).split("/")
        if parts:
            parent = self.site.unrestrictedTraverse("/".join(parts[:-1]))
            self.dest = parent.restrictedTraverse(parts[-1])

        self.catalog = getToolByName(context, "portal_catalog")
        self.mtool = getToolByName(self.context, "portal_membership")

        brains = []
        if keep_selection_order:
            brains = [uuidToCatalogBrain(uid) for uid in selection]
        else:
            brains = self.catalog(UID=selection, show_inactive=True)

        for brain in brains:
            if not brain:
                continue
            # remove everyone so we know if we missed any
            selection.remove(brain.UID)
            obj = brain.getObject()
            if self.required_obj_permission and not self.mtool.checkPermission(
                self.required_obj_permission, obj
            ):
                self.errors.append(
                    _(
                        'Permission denied for "${title}"',
                        mapping={"title": self.objectTitle(obj)},
                    )
                )
                continue
            self.action(obj)

        self.finish()
        return self.message(selection)

    def message(self, missing=[]):
        if len(missing) > 0:
            self.errors.append(
                _("${items} could not be found", mapping={"items": str(len(missing))})
            )
        if self.errors:
            msg = self.failure_msg
        else:
            msg = self.success_msg

        translated_msg = translate(msg, context=self.request)
        if self.errors:
            translated_errors = [
                translate(error, context=self.request) for error in self.errors
            ]
            translated_msg = "{:s}: {:s}".format(
                translated_msg, "\n".join(translated_errors)
            )

        return self.json(
            {"status": "warning" if self.errors else "success", "msg": translated_msg}
        )


@implementer(IFolderContentsView)
class FolderContentsView(BrowserView):
    def get_actions(self):
        actions = []
        for name, Utility in getUtilitiesFor(IStructureAction):
            utility = Utility(self.context, self.request)
            actions.append(utility)
        actions.sort(key=lambda a: a.order)
        return [a.get_options() for a in actions]

    @property
    def ignored_columns(self):
        """Return columns, which should be ignored in folder contents."""
        # These columns either have alternatives or are probably not useful
        ignored = [
            "Date",
            "Title",
            "author_name",
            "cmf_uid",
            "commentators",
            "created",
            "effective",
            "expires",
            "getIcon",
            "getMimeIcon",
            "getId",
            "getRemoteUrl",
            "in_response_to",
            "listCreators",
            "meta_type",
            "modified",
            "portal_type",
            "sync_uid",
        ]
        return ignored

    def get_columns(self):
        columns = {}
        voc = getUtility(IVocabularyFactory, "plone.app.vocabularies.MetadataFields")(
            self.context
        )
        for term in voc:
            if term.value not in self.ignored_columns:
                columns[term.value] = translate(term.title, context=self.request)

        return columns

    def get_thumb_scale(self):
        registry = getUtility(IRegistry)
        settings = registry.forInterface(ISiteSchema, prefix="plone", check=False)
        if settings.no_thumbs_tables:
            # thumbs to be supressed
            return None
        thumb_scale_table = settings.thumb_scale_table
        return thumb_scale_table

    def default_page_types(self):
        registry = getUtility(IRegistry)
        return registry.get("plone.default_page_types", [])

    @property
    def ignored_indexes(self):
        ignored = [
            "Date",
            "Description",
            "Title",
            "allowedRolesAndUsers",
            "author_name",
            "cmf_uid",
            "commentators",
            "effectiveRange",
            "getId",
            "getObjectPositionInParent",
            "getRawRelatedItems",
            "in_reply_to",
            "meta_type",
            "object_provides",
            "portal_type",
            "SearchableText",
            "sync_uid",
        ]
        return ignored

    def get_indexes(self):
        # Base set of indexes
        indexes = {
            "created": translate(_("Created on"), context=self.request),
            "Creator": translate(_("Creator"), context=self.request),
            "effective": translate(_("Publication date"), context=self.request),  # noqa
            "end": translate(_("End Date"), context=self.request),
            "expires": translate(_("Expiration date"), context=self.request),
            "id": translate(_("ID"), context=self.request),
            "is_folderish": translate(_("Folder"), context=self.request),
            "modified": translate(_("Last modified"), context=self.request),  # noqa
            "review_state": translate(_("Review state"), context=self.request),
            "sortable_title": translate(_("Title"), context=self.request),
            "start": translate(_("Start Date"), context=self.request),
            "Subject": translate(_("Tags"), context=self.request),
            "total_comments": translate(
                _("Total comments"), context=self.request
            ),  # noqa
            "Type": translate(_("Type"), context=self.request),
        }
        # Filter out ignored
        indexes = {k: v for k, v in indexes.items() if k not in self.ignored_indexes}
        # Add in extra metadata indexes
        catalog = getToolByName(self.context, "portal_catalog")
        cat_indexes = [idx for idx in catalog.indexes()]
        for index in cat_indexes:
            if index not in indexes and index not in self.ignored_indexes:
                indexes[index] = translate(_(index), context=self.request)
        return indexes

    def get_options(self):
        site = utils.get_top_site_from_url(self.context, self.request)
        base_url = site.absolute_url()
        base_vocabulary = "%s/@@getVocabulary?name=" % base_url
        site_path = site.getPhysicalPath()
        context_path = self.context.getPhysicalPath()
        columns = self.get_columns()
        options = {
            "vocabularyUrl": "%splone.app.vocabularies.Catalog" % (base_vocabulary),
            "urlStructure": {"base": base_url, "appended": "/folder_contents"},
            "moveUrl": "%s{path}/fc-itemOrder" % base_url,
            "indexOptionsUrl": "%s/@@qsOptions" % base_url,
            "contextInfoUrl": "%s{path}/@@fc-contextInfo" % base_url,
            "setDefaultPageUrl": "%s{path}/@@fc-setDefaultPage" % base_url,
            "defaultPageTypes": self.default_page_types(),
            "searchParam": "Title",
            "availableColumns": columns,
            "attributes": [
                "Title",
                "path",
                "getURL",
                "getIcon",
                "getMimeIcon",
                "portal_type",
            ]
            + list(columns.keys()),  # noqa
            "buttons": self.get_actions(),
            "rearrange": {
                "properties": self.get_indexes(),
                "url": "%s{path}/@@fc-rearrange" % base_url,
            },
            "basePath": "/" + "/".join(context_path[len(site_path) :]),
            "upload": {
                "relativePath": "@@fileUpload",
                "baseUrl": base_url,
                "initialFolder": IUUID(self.context, None),
                "useTus": TUS_ENABLED,
            },
            "thumb_scale": self.get_thumb_scale(),
        }
        return options

    def __call__(self):
        self.options = json_dumps(self.get_options())
        return super().__call__()


class ContextInfo(BrowserView):

    attributes = [
        "CreationDate",
        "Creator",
        "Description",
        "EffectiveDate",
        "end",
        "exclude_from_nav",
        "getObjSize",
        "getURL",
        "id",
        "is_folderish",
        "last_comment_date",
        "location",
        "ModificationDate",
        "path",
        "portal_type",
        "review_state",
        "start",
        "Subject",
        "Title",
        "total_comments",
        "Type",
        "UID",
    ]

    def __call__(self):
        factories_menu = getUtility(
            IBrowserMenu, name="plone_contentmenu_factory", context=self.context
        ).getMenuItems(self.context, self.request)
        factories = []
        for item in factories_menu:
            if item.get("title") == "folder_add_settings":
                continue
            title = item.get("title", "")
            factories.append(
                {
                    "id": item.get("id"),
                    "title": title
                    and translate(title, context=self.request)
                    or "",  # noqa
                    "action": item.get("action"),
                }
            )

        context = aq_inner(self.context)
        transform = SafeHTML()
        crumbs = []
        top_site = utils.get_top_site_from_url(self.context, self.request)
        while not context == top_site:
            crumbs.append(
                {
                    "id": context.getId(),
                    "title": transform.scrub_html(
                        utils.pretty_title_or_id(context, context)
                    ),
                }
            )
            context = aq_parent(aq_inner(context))

        catalog = getToolByName(self.context, "portal_catalog")
        try:
            brains = catalog(UID=IUUID(self.context), show_inactive=True)
        except TypeError:
            brains = []
        item = None
        if len(brains) > 0:
            obj = brains[0]
            # context here should be site root
            base_path = "/".join(context.getPhysicalPath())
            item = {}
            for attr in self.attributes:
                key = attr
                if key == "path":
                    attr = "getPath"
                val = getattr(obj, attr, None)
                if callable(val):
                    val = val()
                if key == "path":
                    val = val[len(base_path) :]
                if isinstance(val, (bytes, str)):
                    val = transform.scrub_html(val)
                item[key] = val

        self.request.response.setHeader(
            "Content-Type", "application/json; charset=utf-8"
        )
        return json_dumps(
            {
                "addButtons": factories,
                "defaultPage": self.context.getDefaultPage(),
                "breadcrumbs": [c for c in reversed(crumbs)],
                "object": item,
            }
        )



from zope.interface import Interface


class IFolderContentsView(Interface):
    """Interface, which provides methods for folder contens"""

    def test(a, b, c):
        """A simple replacement of python's test."""

    def getAllowedTypes():
        """Returns allowed types for context."""

    def title():
        """Returns the title for the template."""



from zope.component.testing import setUp
from zope.component.testing import tearDown

import doctest
import unittest


def test_suite():
    return unittest.TestSuite(
        doctest.DocFileSuite(
            "table.txt",
            package="plone.app.content.browser",
            optionflags=doctest.ELLIPSIS,
            setUp=setUp,
            tearDown=tearDown,
        )
    )



from plone.base.utils import safe_text
from plone.batching import Batch
from plone.batching.browser import BatchView
from plone.memoize import instance
from Products.Five.browser.pagetemplatefile import ViewPageTemplateFile
from urllib.parse import quote_plus
from zope.i18nmessageid import MessageFactory
from zope.publisher.browser import BrowserView
from ZTUtils import make_query


_ = MessageFactory("plone")


class TableBatchView(BatchView):
    def make_link(self, pagenumber):
        batchlinkparams = self.request.form.copy()
        return "{}?{}".format(
            self.request.ACTUAL_URL,
            make_query(batchlinkparams, {"pagenumber": pagenumber}),
        )


class Table:
    """
    The table renders a table with sortable columns etc.

    It is meant to be subclassed to provide methods for getting specific table
    info.
    """

    def __init__(
        self,
        request,
        base_url,
        view_url,
        items,
        show_sort_column=False,
        buttons=None,
        pagesize=20,
        show_select_column=True,
        show_size_column=True,
        show_modified_column=True,
        show_status_column=True,
    ):
        if buttons is None:
            buttons = []
        self.request = request
        self.context = None  # Need for view pagetemplate

        self.base_url = base_url
        self.view_url = view_url
        self.url = view_url
        self.items = items
        self.show_sort_column = show_sort_column
        self.show_select_column = show_select_column
        self.show_size_column = show_size_column
        self.show_modified_column = show_modified_column
        self.show_status_column = show_status_column
        self.buttons = buttons
        self.default_page_size = 20
        self.pagesize = pagesize
        self.show_all = request.get("show_all", "").lower() == "true"

        selection = request.get("select")
        if selection == "screen":
            self.selectcurrentbatch = True
        elif selection == "all":
            self.selectall = True

        self.pagenumber = int(request.get("pagenumber", 1))

    def msg_select_item(self, item):
        title_or_id = (
            item.get("title_or_id")
            or item.get("title")
            or item.get("Title")
            or item.get("id")
            or item["getId"]
        )
        return _(
            "checkbox_select_item",
            default="Select ${title}",
            mapping={"title": safe_text(title_or_id)},
        )

    @property
    def within_batch_size(self):
        return len(self.items) <= self.pagesize

    def set_checked(self, item):
        selected = self.selected(item)
        item["checked"] = selected and "checked" or None
        item["table_row_class"] = item.get("table_row_class", "")
        if selected:
            item["table_row_class"] += " selected"

    @property
    @instance.memoize
    def batch(self):
        pagesize = self.pagesize
        if self.show_all:
            pagesize = len(self.items)
        b = Batch.fromPagenumber(
            self.items, pagesize=pagesize, pagenumber=self.pagenumber
        )
        list(map(self.set_checked, b))
        return b

    render = ViewPageTemplateFile("table.pt")

    def batching(self):
        return TableBatchView(self.context, self.request)(self.batch)

    # options
    _selectcurrentbatch = False
    _select_all = False

    def _get_select_currentbatch(self):
        return self._selectcurrentbatch

    def _set_select_currentbatch(self, value):
        self._selectcurrentbatch = value
        if (
            self._selectcurrentbatch
            and self.show_all
            or (len(self.items) <= self.pagesize)
        ):
            self.selectall = True

    selectcurrentbatch = property(_get_select_currentbatch, _set_select_currentbatch)

    def _get_select_all(self):
        return self._select_all

    def _set_select_all(self, value):
        self._select_all = bool(value)
        if self._select_all:
            self._selectcurrentbatch = True

    selectall = property(_get_select_all, _set_select_all)

    @property
    def show_select_all_items(self):
        return self.selectcurrentbatch and not self.selectall

    def get_nosort_class(self):
        """ """
        return "nosort"

    @property
    def selectall_url(self):
        return self.selectnone_url + "&select=all"

    @property
    def selectscreen_url(self):
        return self.selectnone_url + "&select=screen"

    @property
    def selectnone_url(self):
        base = self.view_url + "?pagenumber={}&pagesize={}".format(
            self.pagenumber, self.pagesize
        )
        if self.show_all:
            base += "&show_all=true"
        return base

    @property
    def show_all_url(self):
        return self.view_url + "?show_all=true"

    def selected(self, item):
        if self.selectcurrentbatch:
            return True
        return False

    @property
    def viewname(self):
        return self.view_url.split("?")[0].split("/")[-1]

    def quote_plus(self, string):
        return quote_plus(string)


class TableBrowserView(BrowserView):
    """Base class which can be used from a AJAX view

    Subclasses only need to set the table property to a different
    class."""

    table = None

    def update_table(
        self, pagenumber="1", sort_on="getObjPositionInParent", show_all=False
    ):
        self.request.set("sort_on", sort_on)
        self.request.set("pagenumber", pagenumber)
        table = self.table(
            self.context, self.request, contentFilter={"sort_on": sort_on}
        )
        return table.render()
